@@ -14,23 +14,14 @@
 
 """Support for downloading media from Google APIs."""
 
-import base64
-import hashlib
-import logging
-
 import urllib3.response
 
 from google.resumable_media import _download
 from google.resumable_media import common
-from google.resumable_media.requests import _helpers
+from google.resumable_media import _helpers
+from google.resumable_media.requests import _request_helpers
 
 
-_LOGGER = logging.getLogger(__name__)
-_HASH_HEADER = u"x-goog-hash"
-_MISSING_CHECKSUM = u"""\
-No {checksum_type} checksum was returned from the service while downloading {}
-(which happens for composite objects), so client-side content integrity
-checking is not being performed."""
 _CHECKSUM_MISMATCH = u"""\
 Checksum mismatch while downloading:
 
@@ -46,7 +37,7 @@ but the actual {checksum_type} checksum of the downloaded contents was:
 """
 
 
-class Download(_helpers.RequestsMixin, _download.Download):
+class Download(_request_helpers.RequestsMixin, _download.Download):
     """Helper to manage downloading a resource from a Google API.
 
     "Slices" of the resource can be retrieved by specifying a range
@@ -98,7 +89,7 @@ class Download(_helpers.RequestsMixin, _download.Download):
         # `_get_expected_checksum()` may return None even if a checksum was
         # requested, in which case it will emit an info log _MISSING_CHECKSUM.
         # If an invalid checksum type is specified, this will raise ValueError.
-        expected_checksum, checksum_object = _get_expected_checksum(
+        expected_checksum, checksum_object = _helpers._get_expected_checksum(
             response, self._get_headers, self.media_url, checksum_type=self.checksum
         )
 
@@ -109,7 +100,7 @@ class Download(_helpers.RequestsMixin, _download.Download):
             # object to the decoder and return a _DoNothingHash here.
             local_checksum_object = _add_decoder(response.raw, checksum_object)
             body_iter = response.iter_content(
-                chunk_size=_helpers._SINGLE_GET_CHUNK_SIZE, decode_unicode=False
+                chunk_size=_request_helpers._SINGLE_GET_CHUNK_SIZE, decode_unicode=False
             )
             for chunk in body_iter:
                 self._stream.write(chunk)
@@ -118,10 +109,7 @@ class Download(_helpers.RequestsMixin, _download.Download):
         if expected_checksum is None:
             return
         else:
-            actual_checksum = base64.b64encode(checksum_object.digest())
-            # NOTE: ``b64encode`` returns ``bytes``, but ``expected_checksum``
-            #       came from a header, so it will be ``str``.
-            actual_checksum = actual_checksum.decode(u"utf-8")
+            actual_checksum = _helpers.prepare_checksum_digest(checksum_object.digest())
             if actual_checksum != expected_checksum:
                 msg = _CHECKSUM_MISMATCH.format(
                     self.media_url,
@@ -134,7 +122,10 @@ class Download(_helpers.RequestsMixin, _download.Download):
     def consume(
         self,
         transport,
-        timeout=(_helpers._DEFAULT_CONNECT_TIMEOUT, _helpers._DEFAULT_READ_TIMEOUT),
+        timeout=(
+            _request_helpers._DEFAULT_CONNECT_TIMEOUT,
+            _request_helpers._DEFAULT_READ_TIMEOUT,
+        ),
     ):
         """Consume the resource to be downloaded.
 
@@ -172,7 +163,7 @@ class Download(_helpers.RequestsMixin, _download.Download):
         if self._stream is not None:
             request_kwargs[u"stream"] = True
 
-        result = _helpers.http_request(transport, method, url, **request_kwargs)
+        result = _request_helpers.http_request(transport, method, url, **request_kwargs)
 
         self._process_response(result)
 
@@ -182,7 +173,7 @@ class Download(_helpers.RequestsMixin, _download.Download):
         return result
 
 
-class RawDownload(_helpers.RawRequestsMixin, _download.Download):
+class RawDownload(_request_helpers.RawRequestsMixin, _download.Download):
     """Helper to manage downloading a raw resource from a Google API.
 
     "Slices" of the resource can be retrieved by specifying a range
@@ -233,13 +224,13 @@ class RawDownload(_helpers.RawRequestsMixin, _download.Download):
         # `_get_expected_checksum()` may return None even if a checksum was
         # requested, in which case it will emit an info log _MISSING_CHECKSUM.
         # If an invalid checksum type is specified, this will raise ValueError.
-        expected_checksum, checksum_object = _get_expected_checksum(
+        expected_checksum, checksum_object = _helpers._get_expected_checksum(
             response, self._get_headers, self.media_url, checksum_type=self.checksum
         )
 
         with response:
             body_iter = response.raw.stream(
-                _helpers._SINGLE_GET_CHUNK_SIZE, decode_content=False
+                _request_helpers._SINGLE_GET_CHUNK_SIZE, decode_content=False
             )
             for chunk in body_iter:
                 self._stream.write(chunk)
@@ -249,11 +240,8 @@ class RawDownload(_helpers.RawRequestsMixin, _download.Download):
         if expected_checksum is None:
             return
         else:
-            actual_checksum = base64.b64encode(checksum_object.digest())
+            actual_checksum = _helpers.prepare_checksum_digest(checksum_object.digest())
 
-            # NOTE: ``b64encode`` returns ``bytes``, but ``expected_checksum``
-            #       came from a header, so it will be ``str``.
-            actual_checksum = actual_checksum.decode(u"utf-8")
             if actual_checksum != expected_checksum:
                 msg = _CHECKSUM_MISMATCH.format(
                     self.media_url,
@@ -266,7 +254,10 @@ class RawDownload(_helpers.RawRequestsMixin, _download.Download):
     def consume(
         self,
         transport,
-        timeout=(_helpers._DEFAULT_CONNECT_TIMEOUT, _helpers._DEFAULT_READ_TIMEOUT),
+        timeout=(
+            _request_helpers._DEFAULT_CONNECT_TIMEOUT,
+            _request_helpers._DEFAULT_READ_TIMEOUT,
+        ),
     ):
         """Consume the resource to be downloaded.
 
@@ -295,7 +286,7 @@ class RawDownload(_helpers.RawRequestsMixin, _download.Download):
         """
         method, url, payload, headers = self._prepare_request()
         # NOTE: We assume "payload is None" but pass it along anyway.
-        result = _helpers.http_request(
+        result = _request_helpers.http_request(
             transport,
             method,
             url,
@@ -314,7 +305,7 @@ class RawDownload(_helpers.RawRequestsMixin, _download.Download):
         return result
 
 
-class ChunkedDownload(_helpers.RequestsMixin, _download.ChunkedDownload):
+class ChunkedDownload(_request_helpers.RequestsMixin, _download.ChunkedDownload):
     """Download a resource in chunks from a Google API.
 
     Args:
@@ -345,7 +336,10 @@ class ChunkedDownload(_helpers.RequestsMixin, _download.ChunkedDownload):
     def consume_next_chunk(
         self,
         transport,
-        timeout=(_helpers._DEFAULT_CONNECT_TIMEOUT, _helpers._DEFAULT_READ_TIMEOUT),
+        timeout=(
+            _request_helpers._DEFAULT_CONNECT_TIMEOUT,
+            _request_helpers._DEFAULT_READ_TIMEOUT,
+        ),
     ):
         """Consume the next chunk of the resource to be downloaded.
 
@@ -368,7 +362,7 @@ class ChunkedDownload(_helpers.RequestsMixin, _download.ChunkedDownload):
         """
         method, url, payload, headers = self._prepare_request()
         # NOTE: We assume "payload is None" but pass it along anyway.
-        result = _helpers.http_request(
+        result = _request_helpers.http_request(
             transport,
             method,
             url,
@@ -381,7 +375,7 @@ class ChunkedDownload(_helpers.RequestsMixin, _download.ChunkedDownload):
         return result
 
 
-class RawChunkedDownload(_helpers.RawRequestsMixin, _download.ChunkedDownload):
+class RawChunkedDownload(_request_helpers.RawRequestsMixin, _download.ChunkedDownload):
     """Download a raw resource in chunks from a Google API.
 
     Args:
@@ -412,7 +406,10 @@ class RawChunkedDownload(_helpers.RawRequestsMixin, _download.ChunkedDownload):
     def consume_next_chunk(
         self,
         transport,
-        timeout=(_helpers._DEFAULT_CONNECT_TIMEOUT, _helpers._DEFAULT_READ_TIMEOUT),
+        timeout=(
+            _request_helpers._DEFAULT_CONNECT_TIMEOUT,
+            _request_helpers._DEFAULT_READ_TIMEOUT,
+        ),
     ):
         """Consume the next chunk of the resource to be downloaded.
 
@@ -435,7 +432,7 @@ class RawChunkedDownload(_helpers.RawRequestsMixin, _download.ChunkedDownload):
         """
         method, url, payload, headers = self._prepare_request()
         # NOTE: We assume "payload is None" but pass it along anyway.
-        result = _helpers.http_request(
+        result = _request_helpers.http_request(
             transport,
             method,
             url,
@@ -449,117 +446,6 @@ class RawChunkedDownload(_helpers.RawRequestsMixin, _download.ChunkedDownload):
         return result
 
 
-def _get_expected_checksum(response, get_headers, media_url, checksum_type):
-    """Get the expected checksum and checksum object for the response.
-
-    Args:
-        response (~requests.Response): The HTTP response object.
-        get_headers (callable: response->dict): returns response headers.
-        media_url (str): The URL containing the media to be downloaded.
-        checksum_type Optional(str): The checksum type to read from the headers,
-            exactly as it will appear in the headers (case-sensitive). Must be
-            "md5", "crc32c" or None.
-
-    Returns:
-        Tuple (Optional[str], object): The expected checksum of the response,
-        if it can be detected from the ``X-Goog-Hash`` header, and the
-        appropriate checksum object for the expected checksum.
-    """
-    if checksum_type not in ["md5", "crc32c", None]:
-        raise ValueError("checksum must be ``'md5'``, ``'crc32c'`` or ``None``")
-    elif checksum_type in ["md5", "crc32c"]:
-        headers = get_headers(response)
-        expected_checksum = _parse_checksum_header(
-            headers.get(_HASH_HEADER), response, checksum_label=checksum_type
-        )
-
-        if expected_checksum is None:
-            msg = _MISSING_CHECKSUM.format(
-                media_url, checksum_type=checksum_type.upper()
-            )
-            _LOGGER.info(msg)
-            checksum_object = _DoNothingHash()
-        else:
-            if checksum_type == "md5":
-                checksum_object = hashlib.md5()
-            else:
-                checksum_object = _helpers._get_crc32c_object()
-    else:
-        expected_checksum = None
-        checksum_object = _DoNothingHash()
-
-    return (expected_checksum, checksum_object)
-
-
-def _parse_checksum_header(header_value, response, checksum_label):
-    """Parses the MD5 header from an ``X-Goog-Hash`` value.
-
-    .. _header reference: https://cloud.google.com/storage/docs/\
-                          xml-api/reference-headers#xgooghash
-
-    Expects ``header_value`` (if not :data:`None`) to be in one of the three
-    following formats:
-
-    * ``crc32c=n03x6A==``
-    * ``md5=Ojk9c3dhfxgoKVVHYwFbHQ==``
-    * ``crc32c=n03x6A==,md5=Ojk9c3dhfxgoKVVHYwFbHQ==``
-
-    See the `header reference`_ for more information.
-
-    Args:
-        header_value (Optional[str]): The ``X-Goog-Hash`` header from
-            a download response.
-        response (~requests.Response): The HTTP response object.
-        checksum_label (str): The label of the header value to read, as in the
-            examples above. Typically "md5" or "crc32c"
-
-    Returns:
-        Optional[str]: The expected checksum of the response, if it
-        can be detected from the ``X-Goog-Hash`` header; otherwise, None.
-
-    Raises:
-        ~google.resumable_media.common.InvalidResponse: If there are
-            multiple checksums of the requested type in ``header_value``.
-    """
-    if header_value is None:
-        return None
-
-    matches = []
-    for checksum in header_value.split(u","):
-        name, value = checksum.split(u"=", 1)
-        if name == checksum_label:
-            matches.append(value)
-
-    if len(matches) == 0:
-        return None
-    elif len(matches) == 1:
-        return matches[0]
-    else:
-        raise common.InvalidResponse(
-            response,
-            u"X-Goog-Hash header had multiple ``{}`` values.".format(checksum_label),
-            header_value,
-            matches,
-        )
-
-
-class _DoNothingHash(object):
-    """Do-nothing hash object.
-
-    Intended as a stand-in for ``hashlib.md5`` or a crc32c checksum
-    implementation in cases where it isn't necessary to compute the hash.
-    """
-
-    def update(self, unused_chunk):
-        """Do-nothing ``update`` method.
-
-        Intended to match the interface of ``hashlib.md5`` and other checksums.
-
-        Args:
-            unused_chunk (bytes): A chunk of data.
-        """
-
-
 def _add_decoder(response_raw, checksum):
     """Patch the ``_decoder`` on a ``urllib3`` response.
 
@@ -584,7 +470,7 @@ def _add_decoder(response_raw, checksum):
         return checksum
 
     response_raw._decoder = _GzipDecoder(checksum)
-    return _DoNothingHash()
+    return _helpers._DoNothingHash()
 
 
 class _GzipDecoder(urllib3.response.GzipDecoder):
